
#include <SCE/SCEngine.h>
#include <SDL.h>

#define W 800
#define H 600

int main (void)
{
    int loop = 1;
    SDL_Event ev;
    SCE_SScene *scene = NULL;
    SCE_SModel *model = NULL;
    SCE_SLight *light = NULL;
    float *m = NULL;

    SDL_Init (SDL_INIT_VIDEO);
    SDL_SetVideoMode (W, H, 32, SDL_OPENGL);

    SCE_Init (stderr, 0);

    scene = SCE_Scene_Create ();
    model = SCE_Model_Create ();
    light = SCE_Light_Create ();

    SCE_Model_LoadMesh (model, "suzanne.obj");
    SCE_Light_SetColor (light, 0.9, 0.6, 0.4);

    m = SCE_Node_GetMatrix (SCE_Model_GetNode (model));
    SCE_Matrix4_Scale (m, 1.0/2.0, 1.0/2.0, 1.0);

    m = SCE_Node_GetMatrix (SCE_Light_GetNode (light));
    SCE_Matrix4_Translate (m, 0.0, 4.0, 4.0);

    SCE_Scene_AddModel (scene, model);
    SCE_Scene_AddLight (scene, light);

    while (loop) {
        while (SDL_PollEvent (&ev)) {
            switch (ev.type) {
            case SDL_QUIT: loop = 0; break;
            default:;
            }
        }
        SCE_Scene_Update (scene, NULL, NULL, 0);
        SCE_Scene_Render (scene, NULL, NULL, 0);
        SDL_GL_SwapBuffers ();
        SDL_Delay (50);
    }

    SCE_Light_Delete (light);
    SCE_Model_Delete (model);
    SCE_Scene_Delete (scene);

    SCE_Quit ();
    SDL_Quit ();
    return 0;
}
